package org.luyinbros.widget;

import android.view.View;

import static android.view.View.getDefaultSize;

public class NineLatticeLayoutManager extends ViewManageLayout.LayoutManager {

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        final int childCount = getChildCount();
        if (childCount == 0) {
            setMeasuredDimension(getDefaultSize(getSuggestedMinimumWidth(), widthMeasureSpec),
                    getDefaultSize(getSuggestedMinimumHeight(), heightMeasureSpec));
            return;
        }
        if (childCount == 1) {
            View child = getChildAt(0);
            measureChild(child, widthMeasureSpec, heightMeasureSpec);
            setMeasuredDimension(Math.max(child.getMeasuredWidth(), getSuggestedMinimumWidth()),
                    Math.max(child.getMeasuredHeight(), getSuggestedMinimumHeight()));
        } else if (childCount == 2 || childCount == 4) {
            measureGrid(2, widthMeasureSpec, heightMeasureSpec);
        } else {
            measureGrid(3, widthMeasureSpec, heightMeasureSpec);
        }

    }

    private void measureGrid(int spanCount,
                             int widthMeasureSpec,
                             int heightMeasureSpec) {
        final int childCount = getChildCount();
        int totalRow;
        if (childCount % spanCount != 0) {
            totalRow = childCount / spanCount + 1;
        } else {
            totalRow = childCount / spanCount;
        }
        int widthMode = View.MeasureSpec.getMode(widthMeasureSpec);
        if (widthMode == View.MeasureSpec.AT_MOST || widthMode == View.MeasureSpec.UNSPECIFIED) {
            int totalHeight = 0;
            int rowMaxHeight = 0;
            int totalWidth = 0;
            int columnWidth = 0;
            for (int i = 0; i < childCount; i++) {
                View child = getChildAt(i);
                measureChild(child, widthMeasureSpec,
                        heightMeasureSpec);
                if (i % spanCount == spanCount - 1) {
                    columnWidth += child.getMeasuredWidth();
                    totalHeight += Math.max(rowMaxHeight, child.getMeasuredHeight());
                    totalWidth = Math.max(columnWidth, totalWidth);
                    rowMaxHeight = 0;
                    columnWidth = 0;
                } else {

                    columnWidth += child.getMeasuredWidth();
                    rowMaxHeight = Math.max(rowMaxHeight, child.getMeasuredHeight());
                }
            }
            setMeasuredDimension(Math.max(getSuggestedMinimumWidth(), totalWidth),
                    Math.max(getSuggestedMinimumHeight(), totalHeight));
        } else {
            int parentWidth = View.MeasureSpec.getSize(widthMeasureSpec);
            int squareSize = parentWidth / spanCount;
            for (int i = 0; i < childCount; i++) {
                View child = getChildAt(i);
                child.measure(View.MeasureSpec.makeMeasureSpec(squareSize, View.MeasureSpec.EXACTLY),
                        View.MeasureSpec.makeMeasureSpec(squareSize, View.MeasureSpec.EXACTLY));
            }
            switch (View.MeasureSpec.getMode(heightMeasureSpec)) {
                case View.MeasureSpec.EXACTLY:
                    setMeasuredDimension(parentWidth, Math.min(totalRow * squareSize
                            , View.MeasureSpec.getSize(heightMeasureSpec)));
                    break;
                case View.MeasureSpec.AT_MOST:
                case View.MeasureSpec.UNSPECIFIED:
                default:
                    setMeasuredDimension(parentWidth, totalRow * squareSize);
            }
        }

    }

    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
        final int childCount = getChildCount();
        if (childCount == 1) {
            View child = getChildAt(0);
            child.layout(0, 0, child.getMeasuredWidth(), child.getMeasuredHeight());
        } else if (childCount == 2 || childCount == 4) {
            onLayout(2);
        } else {
            onLayout(3);
        }
    }

    private void onLayout(int spanCount) {
        final int childCount = getChildCount();
        int left = 0;
        int top = 0;
        for (int i = 0; i < childCount; i++) {
            View child = getChildAt(i);
            child.layout(left, top, left + child.getMeasuredWidth(), top + child.getMeasuredHeight());
            if (i % spanCount == spanCount - 1) {
                left = 0;
                top += child.getMeasuredHeight();
            } else {
                left += child.getMeasuredWidth();
            }
        }
    }
}
